@@ -80,9 +80,11 @@
 // This is a private option that should not be used.
 // https://github.com/rust-lang/docs.rs/issues/147#issuecomment-389544407
 #![cfg_attr(process_control_docs_rs, feature(doc_cfg))]
+#![warn(unsafe_op_in_unsafe_fn)]
 #![warn(unused_results)]
 
 use std::fmt;
+use std::fmt::Debug;
 use std::fmt::Display;
 use std::fmt::Formatter;
 use std::io;
@@ -108,13 +110,16 @@ macro_rules! if_memory_limit {
 
 mod control;
 
-#[cfg_attr(unix, path = "unix.rs")]
-#[cfg_attr(windows, path = "windows.rs")]
+#[cfg_attr(unix, path = "unix/mod.rs")]
+#[cfg_attr(windows, path = "windows/mod.rs")]
 mod imp;
 
+type WaitResult<T> = io::Result<Option<T>>;
+
 /// A wrapper that stores enough information to terminate a process.
 ///
 /// Instances can only be constructed using [`ChildExt::terminator`].
+#[deprecated = "cannot be used safely and should be unnecessary"]
 #[derive(Debug)]
 pub struct Terminator(imp::DuplicatedHandle);
 
@@ -162,12 +167,35 @@ impl Terminator {
     /// #
     /// # Ok::<_, io::Error>(())
     /// ```
+    #[deprecated = "cannot be used safely and should be unnecessary"]
     #[inline]
     pub unsafe fn terminate(&self) -> io::Result<()> {
-        self.0.terminate()
+        // SAFETY: The safety requirements are documented.
+        unsafe { self.0.terminate() }
     }
 }
 
+#[rustfmt::skip]
+macro_rules! unix_method {
+    ( $method:ident , $return_type:ty ) => {
+        #[doc = concat!(
+            "Equivalent to [`ExitStatusExt::",
+            stringify!($method),
+            "`][method].
+
+[method]: ::std::os::unix::process::ExitStatusExt::",
+            stringify!($method),
+        )]
+        #[cfg(any(unix, doc))]
+        #[cfg_attr(process_control_docs_rs, doc(cfg(unix)))]
+        #[inline]
+        #[must_use]
+        pub fn $method(&self) -> $return_type {
+            self.0.$method()
+        }
+    };
+}
+
 /// Equivalent to [`process::ExitStatus`] but allows for greater accuracy.
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct ExitStatus(imp::ExitStatus);
@@ -188,22 +216,16 @@ impl ExitStatus {
         self.0.code().map(Into::into)
     }
 
-    /// Equivalent to [`ExitStatusExt::signal`].
-    ///
-    /// [`ExitStatusExt::signal`]: ::std::os::unix::process::ExitStatusExt::signal
-    #[cfg(any(unix, doc))]
-    #[cfg_attr(process_control_docs_rs, doc(cfg(unix)))]
-    #[inline]
-    #[must_use]
-    pub fn signal(self) -> Option<::std::os::raw::c_int> {
-        self.0.signal()
-    }
+    unix_method!(continued, bool);
+    unix_method!(core_dumped, bool);
+    unix_method!(signal, Option<::std::os::raw::c_int>);
+    unix_method!(stopped_signal, Option<::std::os::raw::c_int>);
 }
 
 impl Display for ExitStatus {
     #[inline]
     fn fmt(&self, f: &mut Formatter<'_>) -> fmt::Result {
-        self.0.fmt(f)
+        Display::fmt(&self.0, f)
     }
 }
 
@@ -326,7 +348,7 @@ pub trait Control: private::Sealed {
     /// internally.
     ///
     /// [`terminate_for_timeout`]: Self::terminate_for_timeout
-    fn wait(self) -> io::Result<Option<Self::Result>>;
+    fn wait(self) -> WaitResult<Self::Result>;
 }
 
 /// A temporary wrapper for a process timeout.
@@ -378,7 +400,7 @@ pub trait Timeout: private::Sealed {
     ///
     /// [`terminating`]: Self::terminating
     #[deprecated = "use `Control::wait` instead"]
-    fn wait(self) -> io::Result<Option<Self::Result>>;
+    fn wait(self) -> WaitResult<Self::Result>;
 }
 
 /// Extensions to [`Child`] for easily terminating processes.
@@ -388,12 +410,12 @@ pub trait ChildExt<'a>: private::Sealed {
     /// The type returned by [`controlled`].
     ///
     /// [`controlled`]: Self::controlled
-    type ExitStatusControl: 'a + Control<Result = ExitStatus>;
+    type ExitStatusControl: 'a + Control<Result = ExitStatus> + Debug;
 
     /// The type returned by [`controlled_with_output`].
     ///
     /// [`controlled_with_output`]: Self::controlled_with_output
-    type OutputControl: Control<Result = Output>;
+    type OutputControl: Control<Result = Output> + Debug;
 
     /// The type returned by [`with_timeout`].
     ///
@@ -422,8 +444,17 @@ pub trait ChildExt<'a>: private::Sealed {
     /// #
     /// # Ok::<_, io::Error>(())
     /// ```
+    #[deprecated = "cannot be used safely and should be unnecessary"]
     fn terminator(&self) -> io::Result<Terminator>;
 
+    /// Equivalent to [`Child::kill`] but ignores errors when the process is no
+    /// longer running.
+    ///
+    /// Windows and Unix errors are inconsistent when terminating processes.
+    /// This method unifies them by simulating Unix behavior on Windows.
+    #[allow(clippy::missing_errors_doc)]
+    fn terminate_if_running(&mut self) -> io::Result<()>;
+
     /// Creates an instance of [`Control`] that yields [`ExitStatus`] for this
     /// process.
     ///
@@ -563,6 +594,11 @@ impl<'a> ChildExt<'a> for Child {
         imp::DuplicatedHandle::new(self).map(Terminator)
     }
 
+    #[inline]
+    fn terminate_if_running(&mut self) -> io::Result<()> {
+        imp::terminate_if_running(self)
+    }
+
     #[inline]
     fn controlled(&'a mut self) -> Self::ExitStatusControl {
         Self::ExitStatusControl::new(self)
